@@ -46,11 +46,16 @@ extern "C" {
 #include <Lum/Def/Menu.h>
 #include <Lum/Def/MultiView.h>
 
+#include <Lum/Images/Image.h>
+#include <Lum/Images/Loader.h>
+
+#include <Lum/Manager/Behaviour.h>
+#include <Lum/Manager/Display.h>
+#include <Lum/Manager/FileSystem.h>
+
 #include <Lum/OS/Probe.h>
 #include <Lum/OS/Thread.h>
 
-#include <Lum/OS/Manager/Repository.h>
-
 #include <Lum/Base/DateTime.h>
 #include <Lum/Base/L10N.h>
 #include <Lum/Base/Path.h>
@@ -59,6 +64,7 @@ extern "C" {
 #include <Lum/Dlg/About.h>
 #include <Lum/Dlg/File.h>
 #include <Lum/Dlg/Msg.h>
+#include <Lum/Dlg/Properties.h>
 #include <Lum/Dlg/Value.h>
 
 #include <Lum/Model/Action.h>
@@ -67,8 +73,7 @@ extern "C" {
 #include <Lum/Model/Header.h>
 #include <Lum/Model/String.h>
 
-#include <Lum/OS/Main.h>
-
+#include <Lum/Application.h>
 #include <Lum/Button.h>
 #include <Lum/Dialog.h>
 #include <Lum/Label.h>
@@ -82,7 +87,6 @@ extern "C" {
 #include <Lum/View.h>
 
 #include "Configuration.h"
-#include "Settings.h"
 
 #include "GPSData.h"
 #include "Compass.h"
@@ -150,53 +154,31 @@ double GetRelativeQualityFromDbm(double quality)
   }
 }
 
-class SatelliteObject : public Lum::Object
+class SatellitesModelPainter : public Lum::TableCellPainter
 {
 private:
-  Lum::OS::FontRef   font;
   Lum::OS::ColorRef  green;
   Lum::OS::ColorRef  grey;
-  GPSData::Satellite satellite;
 
 public:
-  SatelliteObject()
-  : font(Lum::OS::display->GetFont()),
-    green(0,1,0,Lum::OS::display->GetColor(Lum::OS::Display::tableTextColor)),
-    grey(0.7,0.7,0.7,Lum::OS::display->GetColor(Lum::OS::Display::tableTextColor))
+  SatellitesModelPainter()
+    : green(0,1,0,Lum::OS::display->GetColor(Lum::OS::Display::tableTextColor)),
+      grey(0.7,0.7,0.7,Lum::OS::display->GetColor(Lum::OS::Display::tableTextColor))
   {
     // no code
   }
 
-  void SetSatellite(const GPSData::Satellite& satellite)
+  size_t GetProposedHeight() const
   {
-    this->satellite=satellite;
+    return Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder)+
+           GetFont()->height+
+           Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder);
   }
 
-  void CalcSize()
+  void Draw(Lum::OS::DrawInfo* draw,
+            int x, int y, size_t width, size_t height) const
   {
-    minWidth=Lum::OS::display->GetSpaceHorizontal(Lum::OS::Display::spaceObjectBorder)+
-             10*font->StringWidth(L"m");
-    minHeight=Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder)+
-              font->height+
-              Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder);
-
-    width=minWidth;
-    height=minHeight;
-
-    Object::CalcSize();
-  }
-
-  void Draw(int x, int y, size_t w, size_t h)
-  {
-    Object::Draw(x,y,w,h);
-
-    if (!OIntersect(x,y,w,h)) {
-      return;
-    }
-
-    /* --- */
-
-    Lum::OS::DrawInfo *draw=GetDrawInfo();
+    const GPSData::Satellite& satellite=dynamic_cast<const SatellitesModel*>(GetModel())->GetEntry(GetRow());
 
     //
     // Signal strength bar
@@ -211,8 +193,8 @@ public:
       draw->PushForeground(grey);
     }
 
-    draw->FillRectangle(this->x,
-                        this->y+
+    draw->FillRectangle(x,
+                        y+
                         Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder),
                         (size_t)(width*percent),height-2);
 
@@ -223,10 +205,10 @@ public:
     // Name of Satellite
     //
 
-    draw->PushFont(font);
-    draw->PushForeground(parent->GetTextColor(draw));
-    draw->DrawString(this->x,
-                     this->y+font->ascent,
+    draw->PushFont(GetFont());
+    draw->PushForeground(GetTextColor(draw));
+    draw->DrawString(x,
+                     y+GetFont()->ascent,
                      Lum::Base::NumberToWString(satellite.id));
     draw->PopForeground();
     draw->PopFont();
@@ -235,55 +217,21 @@ public:
     //
     // Signal strength label
 
-    Lum::OS::FontRef    font=Lum::OS::display->GetFont();
     std::wstring        label=Lum::Base::NumberToWString(lround(percent*100))+L"%";
     Lum::OS::FontExtent extent;
 
-    font->StringExtent(label,extent);
+    GetFont()->StringExtent(label,extent);
 
     draw->PushForeground(Lum::OS::Display::textColor);
-    draw->PushFont(font,0);
-    draw->DrawString(this->x+(width-extent.width+extent.left+extent.right)/2,
-                     this->y+
+    draw->PushFont(GetFont(),Lum::OS::Font::normal);
+    draw->DrawString(x+(width-extent.width+extent.left+extent.right)/2,
+                     y+
                      Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder)+
-                     font->ascent,
+                     GetFont()->ascent,
                      label);
     draw->PopFont();
     draw->PopForeground();
   }
-
-  static SatelliteObject* Create(bool horizFlex, bool vertFlex)
-  {
-    SatelliteObject *object=new SatelliteObject();
-
-    object->SetFlex(horizFlex,vertFlex);
-
-    return object;
-  }
-};
-
-class SatellitesDataProvider : public SatellitesModel::DataProvider
-{
-private:
-  SatelliteObject *object;
-
-public:
-  SatellitesDataProvider()
-  : object(SatelliteObject::Create(true,false))
-  {
-    // no code
-  }
-
-  ~SatellitesDataProvider()
-  {
-    delete object;
-  }
-
-  Lum::Object* GetObject(const SatellitesModel::Iterator& iter, size_t column) const
-  {
-    object->SetSatellite(*iter);
-    return object;
-  }
 };
 
 class MyWindow;
@@ -293,10 +241,16 @@ static unsigned long     trackingFileId='G'*256*256*256+'J'*256*256+'T'*256+'\0'
 static unsigned long     trackingFileVersion=2;
 
 #if defined(HAVE_LIB_GPS)
-static void cb( struct gps_data_t * data,
+  #if GPSD_API_MAJOR_VERSION>=4
+static void cb(struct gps_data_t * data,
+               char * buf,
+               size_t len);
+  #else
+static void cb(struct gps_data_t * data,
                char * buf,
                size_t len,
-               int level );
+               int level);
+  #endif
 #elif defined(HAVE_LIB_LOCATION)
 static void gps_location_changed(LocationGPSDevice *device, gpointer userdata);
 static void gps_location_connected(LocationGPSDevice *new_device, gpointer userdata);
@@ -336,106 +290,33 @@ public:
   }
 };
 
-class TrackObject : public Lum::Object
+typedef Lum::Model::StdTable<Track> TracksModel;
+typedef Lum::Base::Reference<TracksModel> TracksModelRef;
+
+class TracksModelPainter : public Lum::TableCellPainter
 {
-private:
-  Lum::OS::FontRef font;
-  Track            track;
-
 public:
-  TrackObject()
-  : font(Lum::OS::display->GetFont())
+  void Draw(Lum::OS::DrawInfo* draw,
+            int x, int y, size_t width, size_t height) const
   {
-    // no code
-  }
-
-  void SetTrack(const Track& track)
-  {
-    this->track=track;
-  }
-
-  void CalcSize()
-  {
-    minWidth=20*font->StringWidth(L"m")+
-             Lum::OS::display->GetSpaceHorizontal(Lum::OS::Display::spaceObjectBorder)+
-             5*font->StringWidth(L"m");
-    minHeight=font->height;
-
-    width=minWidth;
-    height=minHeight;
-
-    Object::CalcSize();
-  }
-
-  void Draw(int x, int y, size_t w, size_t h)
-  {
-    Object::Draw(x,y,w,h);
-
-    if (!OIntersect(x,y,w,h)) {
-      return;
-    }
-
-    /* --- */
-
-    Lum::OS::DrawInfo *draw=GetDrawInfo();
-
-    draw->PushFont(font);
-    draw->PushForeground(parent->GetTextColor(draw));
+    const Track& track=dynamic_cast<const TracksModel*>(GetModel())->GetEntry(GetRow());
 
     //
     // Name of Track
     //
 
-    draw->DrawString(this->x,
-                     this->y+font->ascent,
-                     track.GetName());
+
+    DrawStringLeftAligned(draw,
+                           x,y,width,height,
+                           track.GetName());
 
     //
     //
     // Time of track
 
-    draw->DrawString(this->x+width-font->StringWidth(track.GetTimeString()),
-                     this->y+font->ascent,
-                     track.GetTimeString());
-
-    draw->PopFont();
-    draw->PopForeground();
-  }
-
-  static TrackObject* Create(bool horizFlex, bool vertFlex)
-  {
-    TrackObject *object=new TrackObject();
-
-    object->SetFlex(horizFlex,vertFlex);
-
-    return object;
-  }
-};
-
-typedef Lum::Model::StdTable<Track> TracksModel;
-typedef Lum::Base::Reference<TracksModel> TracksModelRef;
-
-class TracksDataProvider : public TracksModel::DataProvider
-{
-private:
-  TrackObject *object;
-
-public:
-  TracksDataProvider()
-   : object(TrackObject::Create(true,false))
-  {
-    // no code
-  }
-
-  ~TracksDataProvider()
-  {
-    delete object;
-  }
-
-  Lum::Object* GetObject(const TracksModel::Iterator& iter, size_t column) const
-  {
-    object->SetTrack(*iter);
-    return object;
+    DrawStringRightAligned(draw,
+                           x,y,width,height,
+                           track.GetTimeString());
   }
 };
 
@@ -502,7 +383,7 @@ private:
     guint                  idd_changed;
     guint                  idd_con;
     guint                  idd_disc;
-	
+
     guint                  idc_run;
     guint                  idc_stop;
     guint                  idc_error;
@@ -556,7 +437,7 @@ private:
                    NULL);
       g_object_set(G_OBJECT(control),
                    "preferred-interval",
-                   LOCATION_INTERVAL_5S,
+                   LOCATION_INTERVAL_1S,
                    NULL);
 #endif
       std::cout << "Starting GPS daemon..." << std::endl;
@@ -574,8 +455,13 @@ private:
     gpsData=::gps_open("127.0.0.1","2947");
 
     if (gpsData!=NULL) {
+#if GPSD_API_MAJOR_VERSION>=4
+      gps_set_raw_hook(gpsData,cb);
+      gps_stream(gpsData,WATCH_ENABLE,&gpsThread);
+#else
       gps_query(gpsData,"j=0w+x\n");
       gps_set_callback(gpsData,cb,&gpsThread);
+#endif
       std::cout << "GPS initialized!" << std::endl;
     }
     else {
@@ -600,13 +486,13 @@ private:
       g_signal_handler_disconnect(device,idd_changed);
       g_signal_handler_disconnect(device,idd_con);
       g_signal_handler_disconnect(device,idd_disc);
-  	
+
       g_signal_handler_disconnect(control,idc_run);
       g_signal_handler_disconnect(control,idc_stop);
       g_signal_handler_disconnect(control,idc_error);
 
       std::cout << "Freeing LocationGPSDevice..." << std::endl;
-      g_object_unref(device);	
+      g_object_unref(device);
     }
 #endif
 
@@ -639,8 +525,13 @@ private:
     gpsData=::gps_open("127.0.0.1","2947");
 
     if (gpsData!=NULL) {
+#if GPSD_API_MAJOR_VERSION>=4
+      gps_set_raw_hook(gpsData,cb);
+      gps_stream(gpsData,WATCH_ENABLE,&gpsThread);
+#else
       gps_query(gpsData,"j=0w+x\n");
       gps_set_callback(gpsData,cb,&gpsThread);
+#endif
       data.SetConnected(true);
       std::cout << "GPS initialized!" << std::endl;
     }
@@ -665,10 +556,10 @@ public:
     track(new Lum::Model::String(L"")),
     direction(new Lum::Model::Double()),
     climb(new Lum::Model::String(L"")),
-    satellites(new SatellitesModel(new SatellitesDataProvider())),
+    satellites(new SatellitesModel()),
     speedStream(new Lum::Model::DoubleDataStream()),
     altitudeStream(new Lum::Model::DoubleDataStream()),
-    tracks(new TracksModel(new TracksDataProvider(),new TrackComparator())),
+    tracks(new TracksModel(new TrackComparator())),
     trackSelection(new Lum::Model::SingleLineSelection()),
     startTrackingAction(new Lum::Model::Action()),
     stopTrackingAction(new Lum::Model::Action()),
@@ -710,6 +601,7 @@ public:
     Observe(trackSelection);
 
     Observe(unitMode);
+    Observe(compassShowRawGPSDirection);
     Observe(prohibitScreenBlanking);
 
     ::window=this;
@@ -781,6 +673,7 @@ public:
     table->SetShowHeader(false);
     table->SetHeaderModel(headerModel);
     table->SetModel(satellites);
+    table->SetPainter(new SatellitesModelPainter());
     table->GetTableView()->SetAutoFitColumns(true);
 
     multiView.AddView(Lum::Def::MultiView::View(1,
@@ -793,7 +686,7 @@ public:
 
     multiView.AddView(Lum::Def::MultiView::View(3,
                                                 Lum::Def::Desc(L"Compass"),
-                                                Compass::Create(direction,true,true)));
+                                                Compass::Create(direction,true,true,compassShowCourseDirection)));
 
     multiView.AddView(Lum::Def::MultiView::View(4,
                                                 Lum::Def::Desc(L"Diagrams"),
@@ -815,31 +708,28 @@ public:
     table->SetHeaderModel(headerModel);
     table->SetModel(tracks);
     table->SetSelection(trackSelection);
+    table->SetPainter(new TracksModelPainter());
     table->GetTableView()->SetAutoFitColumns(true);
 
     multiView.AddView(Lum::Def::MultiView::View(5,
                                                 Lum::Def::Desc(L"Tracking"),
                                                 Lum::VPanel::Create(true,true)
                                                 ->Add(Lum::HPanel::Create(true,false)
-                                                      ->Add(Lum::Button::Create(L"Record",
-                                                                                Lum::OS::Theme::imageMediaRecord,
-                                                                                startTrackingAction,
-                                                                                Lum::Button::typeIcon))
-                                                      ->Add(Lum::Button::Create(L"Stop",
-                                                                                Lum::OS::Theme::imageMediaStop,
-                                                                                stopTrackingAction,
-                                                                                Lum::Button::typeIcon))
+                                                      ->Add(Lum::Manager::Behaviour::Instance()->GetActionControl(Lum::Def::Action(Lum::Def::Desc(L"_Record"),
+                                                                                                                                   startTrackingAction)
+                                                                                                                  .SetImage(L"Record.png")))
+                                                      ->Add(Lum::Manager::Behaviour::Instance()->GetActionControl(Lum::Def::Action(Lum::Def::Desc(L"_Stop"),
+                                                                                                                                   stopTrackingAction)
+                                                                                                                  .SetImage(L"Stop.png")))
                                                       ->AddSpace(true)
                                                        ->Add(new Lum::LED(tracking))
                                                        ->AddSpace(true)
-                                                      ->Add(Lum::Button::Create(L"_Delete",
-                                                                                Lum::OS::Theme::imageRemove,
-                                                                                deleteTrackAction,
-                                                                                Lum::Button::typeIcon))
-                                                      ->Add(Lum::Button::Create(L"_Export",
-                                                                                Lum::OS::Theme::imageSave,
-                                                                                exportTrackAction,
-                                                                                Lum::Button::typeIcon)))
+                                                      ->Add(Lum::Manager::Behaviour::Instance()->GetActionControl(Lum::Def::Action(Lum::Def::Desc(L"_Delete"),
+                                                                                                                                   deleteTrackAction)
+                                                                                                                  .SetImage(L"Delete.png")))
+                                                      ->Add(Lum::Manager::Behaviour::Instance()->GetActionControl(Lum::Def::Action(Lum::Def::Desc(L"_Export"),
+                                                                                                                                   exportTrackAction)
+                                                                                                                  .SetImage(L"Save.png"))))
                                                 ->AddSpace()
                                                 ->Add(table)));
 
@@ -863,7 +753,7 @@ public:
 
     multiView.SetMenu(preMenu,postMenu);
 
-    Lum::OS::display->GetBehaviour()->ApplyMultiViewDlg(this,multiView);
+    Lum::Manager::Behaviour::Instance()->ApplyMultiViewDlg(this,multiView);
 
     Dialog::PreInit();
   }
@@ -878,6 +768,7 @@ public:
   void StartTracking()
   {
     Lum::Model::StringRef trackName=new Lum::Model::String(L"");
+    std::wstring          homeDir;
     Lum::Base::Path       fileName;
 
     if (!Lum::Dlg::TextInput::GetText(GetWindow(),
@@ -887,7 +778,12 @@ public:
       return;
     }
 
-    fileName.SetNativeDir(Lum::Base::Path::GetHome());
+    if (!Lum::Manager::FileSystem::Instance()->GetEntry(Lum::Manager::FileSystem::userDir,
+        homeDir)) {
+      return;
+    }
+
+    fileName.SetNativeDir(homeDir);
     fileName.AppendDir(L".gpsjinni");
 
     Lum::Base::Path::CreateDir(fileName.GetPath());
@@ -970,8 +866,6 @@ public:
     unsigned long falt=htonl((unsigned long)rint(alt*100));
     unsigned long ftim=htonl(tim.GetTime());
 
-    //std::cout << "TrackRecord: " << std::hex << flon << " " << flat << " " << falt << " " << ftim << std::endl;
-
     trackFile.write((const char*)&flon,sizeof(flon));
     trackFile.write((const char*)&flat,sizeof(flat));
     trackFile.write((const char*)&falt,sizeof(falt));
@@ -995,11 +889,17 @@ public:
     Lum::Base::DirScannerRef     scanner;
     Lum::Base::DirScanner::Entry entry;
     Lum::Base::Status            status;
+    std::wstring                 homeDir;
 
     tracks->Clear();
     tracks->Off();
 
-    dirName.SetNativeDir(Lum::Base::Path::GetHome());
+    if (!Lum::Manager::FileSystem::Instance()->GetEntry(Lum::Manager::FileSystem::userDir,
+        homeDir)) {
+      return;
+    }
+
+    dirName.SetNativeDir(homeDir);
     dirName.AppendDir(L".gpsjinni");
 
     scanner=Lum::Base::DirScanner::Create(dirName);
@@ -1081,8 +981,14 @@ public:
     Lum::Model::Dir::PatternFilter *filter;
     std::ifstream                  in;
     std::ofstream                  out;
+    std::wstring                   homeDir;
 
-    path.SetNativeDir(Lum::Base::Path::GetHome());
+    if (!Lum::Manager::FileSystem::Instance()->GetEntry(Lum::Manager::FileSystem::userDir,
+        homeDir)) {
+      return;
+    }
+
+    path.SetNativeDir(homeDir);
     path.AppendDir(L".gpsjinni");
     path.SetBaseName(name+L".gjt");
 
@@ -1139,17 +1045,19 @@ public:
       return;
     }
 
-    std::wstring envdir;
+    std::cout << "Track data: " << std::hex << tfid << std::dec << " " << tfv << " " << tft << std::endl;
 
-    envdir=Lum::Base::Path::GetDirFromEnvironment("MYDOCSDIR");
-    if (!envdir.empty()) {
-      path.SetNativeDir(envdir);
-    }
-    else {
-      path.SetNativeDir(Lum::Base::Path::GetHome());
+    std::wstring docsDir;
+
+    if (!Lum::Manager::FileSystem::Instance()->GetEntry(Lum::Manager::FileSystem::userDocumentDir,
+                                                        docsDir)) {
+      if (!Lum::Manager::FileSystem::Instance()->GetEntry(Lum::Manager::FileSystem::userDir,
+                                                          docsDir)) {
+        return;
+      }
     }
 
-    path.SetBaseName(name+L".kml");
+    path.SetNativeDirAndFile(docsDir,name+L".kml");
 
     file=path.GetPath();
 
@@ -1167,8 +1075,6 @@ public:
       return;
     }
 
-    std::cout << "Saving to '" << Lum::Base::WStringToString(file) << "'" << std::endl;
-
     out.open(Lum::Base::WStringToString(file).c_str(),
              std::ios::out|std::ios::trunc);
 
@@ -1207,6 +1113,8 @@ public:
         falt=ntohl(falt);
         ftim=ntohl(ftim);
 
+        //std::cout << flon/10000000.0 << " " << flat/10000000.0 << " " << falt/100.0 << " " << ftim << std::endl;
+
         if (tfv==1) {
           dataPoint.lon=flon/10000000.0-90.0;
           dataPoint.lat=flat/10000000.0-180.0;
@@ -1215,7 +1123,7 @@ public:
           dataPoint.lon=flon/10000000.0-180.0;
           dataPoint.lat=flat/10000000.0-90.0;
         }
-        dataPoint.alt=falt/100;
+        dataPoint.alt=falt/100.0;
         dataPoint.time.SetTime(ftim);
 
         data.push_back(dataPoint);
@@ -1377,8 +1285,14 @@ public:
   {
     Lum::Base::Path   path;
     Lum::Base::Status status;
+    std::wstring      homeDir;
 
-    path.SetNativeDir(Lum::Base::Path::GetHome());
+    if (!Lum::Manager::FileSystem::Instance()->GetEntry(Lum::Manager::FileSystem::userDir,
+                                                        homeDir)) {
+      return;
+    }
+
+    path.SetNativeDir(homeDir);
     path.AppendDir(L".gpsjinni");
     path.SetBaseName(name+L".gjt");
 
@@ -1397,6 +1311,51 @@ public:
     }
   }
 
+  void SetCompassDirectionRaw(double newDirection, double newDeviation)
+  {
+    direction->Set(newDirection); // set to whatever it is.
+    // TODO: eliminate hack to pass deviation to compass:
+    if (finite(newDeviation)) {
+      direction->SetRange(0, 360 + newDeviation);
+    }
+  }
+
+  void SetCompassDirectionWeighted(double newDirection, double newDeviation)
+  {
+    // Check if old and new direction values are available and valid
+
+    double oldDirection;
+
+    if (isnan(newDirection) ||
+        isnan(newDeviation) ||
+        direction->IsNull() ||
+        isnan(oldDirection = direction->Get())
+        ) {
+      SetCompassDirectionRaw(newDirection, newDeviation);
+      return;
+    }
+
+    if (newDeviation < 0) {     // just in case...
+      newDeviation = -newDeviation;
+    }
+
+    // Ignore new direction if deviation is too high; leave old direction valid.
+
+    if (newDeviation > 180) {
+      return;
+    }
+
+    // Weighten the new direction by its accuracy (that is inverse deviation)
+    // in comparison to the old direction is a first experiment...
+    // TODO: find better methods to find and eliminate wild shot values.
+    newDirection = CalculateWeightedDirection(
+                      oldDirection,
+                      newDirection,
+                      1 - (newDeviation / 360),    // normalized weight in range 0..1.,
+                      360);
+    SetCompassDirectionRaw(newDirection, newDeviation);
+  }
+
   void ShowChange(const GPSData& data)
   {
     if (data.HasConnected()) {
@@ -1435,9 +1394,14 @@ public:
     }
 
     if (data.HasLatitude()) {
-      std::wstring l=DoubleToString(data.GetLatitude());
+      std::wstring l;
+      l=GetAngleAsDegreeMinuteSecondString(data.GetLatitude(), L"S ", L"N ", 3);
 
-      l+=+L"\u00b0";
+      l+=L" = ";
+
+      l+=DoubleToString(data.GetLatitude(), 5);
+
+      l+=L"\u00b0";
 
       if (data.GetLatitudeDev()!=0 &&
           finite(data.GetLatitudeDev())) {
@@ -1451,7 +1415,12 @@ public:
     }
 
     if (data.HasLongitude()) {
-      std::wstring l=DoubleToString(data.GetLongitude());
+      std::wstring l;
+      l=GetAngleAsDegreeMinuteSecondString(data.GetLongitude(), L"W ", L"E ", 3);
+
+      l+=L" = ";
+
+      l+=DoubleToString(data.GetLongitude(), 5);
 
       l+=+L"\u00b0";
 
@@ -1478,7 +1447,7 @@ public:
       double       speed=data.GetSpeed();
       double       speedDev=data.GetSpeedDev();
       std::wstring s;
-      
+
       // We expect m/s here
       s=GetSpeedAndUnitAsString(speed,speedDev);
 
@@ -1502,7 +1471,12 @@ public:
 
       track->Set(t);
 
-      direction->Set(data.GetTrack());
+      if (compassShowRawGPSDirection->Get()) {
+        SetCompassDirectionRaw(data.GetTrack(), data.GetTrackDev());
+      }
+      else {
+        SetCompassDirectionWeighted(data.GetTrack(), data.GetTrackDev());
+      }
     }
     else {
       track->SetNull();
@@ -1587,43 +1561,55 @@ public:
 
   void ShowSettings()
   {
-    Settings* settings;
+    Lum::Def::PropGroup *props=new Lum::Def::PropGroup();
 
-    settings=new Settings();
-    settings->SetParent(this);
-    if (settings->Open()) {
-      settings->SetExitAction(settings->GetClosedAction());
-      settings->EventLoop();
-      settings->Close();
-    }
+    Lum::Def::OneOfN    unitModeDef(Lum::Def::Desc(L"Unit mode"),
+                                   ::unitMode);
+    unitModeDef.AddChoice(Lum::Def::OneOfN::Choice(Lum::Def::Desc(L"meter/kilometer")));
+    unitModeDef.AddChoice(Lum::Def::OneOfN::Choice(Lum::Def::Desc(L"feet/miles")));
 
-    delete settings;
+    props->Boolean(Lum::Def::Boolean(Lum::Def::Desc(L"Prohibit screen blanking"),
+                                     ::prohibitScreenBlanking));
+    props->Boolean(Lum::Def::Boolean(Lum::Def::Desc(L"Compass shows course"),
+                                     ::compassShowCourseDirection));
+    props->Boolean(Lum::Def::Boolean(Lum::Def::Desc(L"Compass shows raw GPS data"),
+                                     ::compassShowRawGPSDirection));
+    props->OneOfN(unitModeDef);
+
+    Lum::Dlg::Properties::Show(this,props);
+
+    delete props;
   }
 
   void Resync(Lum::Base::Model* model, const Lum::Base::ResyncMsg& msg)
   {
-    if (model==GetOpenedAction() && GetOpenedAction()->IsFinished()) {
+    if (model==GetOpenedAction() &&
+        GetOpenedAction()->IsFinished()) {
       InitializeGPS();
       Lum::OS::display->AddTimer(refreshTime,0,refreshTimer);
       Lum::OS::display->AddTimer(reconnectTime,0,reconnectCheckTimer);
       RefreshTracks();
       LoadConfig();
     }
-    else if (model==GetClosedAction() && GetClosedAction()->IsFinished()) {
+    else if (model==GetClosedAction() &&
+        GetClosedAction()->IsFinished()) {
       DeinitializeGPS();
       SaveConfig();
     }
-    else if (model==aboutAction && aboutAction->IsFinished()) {
+    else if (model==aboutAction &&
+             aboutAction->IsFinished()) {
       Lum::Dlg::About::Show(this,info);
     }
-    else if (model==dataChangedAction && dataChangedAction->IsFinished()) {
+    else if (model==dataChangedAction &&
+             dataChangedAction->IsFinished()) {
       GPSData dataCopy;
 
       GetGPSDataCopy(dataCopy);
       AddTrackingRecord(dataCopy);
       ShowChange(dataCopy);
     }
-    else if (model==refreshTimer && refreshTimer->IsFinished()) {
+    else if (model==refreshTimer &&
+             refreshTimer->IsFinished()) {
       GPSData dataCopy;
 
       GetGPSDataCopy(dataCopy);
@@ -1633,7 +1619,8 @@ public:
 
       Lum::OS::display->AddTimer(refreshTime,0,refreshTimer);
     }
-    else if (model==reconnectCheckTimer && reconnectCheckTimer->IsFinished()) {
+    else if (model==reconnectCheckTimer &&
+             reconnectCheckTimer->IsFinished()) {
       ReconnectIfDisconnected();
 
       Lum::OS::display->AddTimer(reconnectTime,0,reconnectCheckTimer);
@@ -1668,11 +1655,15 @@ public:
         exportTrackAction->Disable();
       }
     }
-    else if (model==settingsAction && settingsAction->IsFinished()) {
+    else if (model==settingsAction &&
+             settingsAction->IsFinished()) {
       ShowSettings();
     }
     else if (model==prohibitScreenBlanking) {
-      Lum::OS::Manager::DisplayManager *displayManager=Lum::OS::Manager::repository->GetDisplayManager();
+      Lum::Manager::Display *displayManager=Lum::Manager::Display::Instance();
+
+      std::cout << "prohibitScreenBlanking changed! displayManager "
+                << (displayManager ? "!= NULL" : "== NULL") << std::endl;
 
       if (displayManager!=NULL) {
         displayManager->AllowScreenBlanking(!prohibitScreenBlanking->Get());
@@ -1686,6 +1677,13 @@ public:
       ShowChange(dataCopy);
       //UpdateStreams(dataCopy);
     }
+    else if (model==compassShowRawGPSDirection) {
+      std::cout << "compassShowRawGPSDirection changed!" << std::endl;
+      GPSData dataCopy;
+
+      GetGPSDataCopy(dataCopy);
+      ShowChange(dataCopy);
+    }
 
     Dialog::Resync(model,msg);
   }
@@ -1697,7 +1695,11 @@ public:
 };
 
 #if defined(HAVE_LIB_GPS)
+  #if GPSD_API_MAJOR_VERSION>=4
+static void cb(struct gps_data_t* data, char * buf, size_t len)
+  #else
 static void cb(struct gps_data_t* data, char * buf, size_t len, int level)
+  #endif
 {
   assert(window!=NULL);
 
@@ -1732,7 +1734,7 @@ static void cb(struct gps_data_t* data, char * buf, size_t len, int level)
   }
 
   if ((data->set & TRACK_SET) && !isnan(data->fix.track)) {
-    gpsData.SetTrack(data->fix.track,data->fix,epd);
+    gpsData.SetTrack(data->fix.track,data->fix.epd);
   }
 
   if ((data->set & CLIMB_SET) && !isnan(data->fix.climb)) {
@@ -1747,7 +1749,7 @@ static void cb(struct gps_data_t* data, char * buf, size_t len, int level)
   if (data->set & SATELLITE_SET) {
     std::vector<GPSData::Satellite> satellites;
 
-    for (int i=0; i<data->satellites; i++) {
+    for (int i=0; i<data->satellites_used; i++) {
       GPSData::Satellite s;
 
       s.id=data->PRN[i];
@@ -1844,11 +1846,13 @@ static void gps_location_error(LocationGPSDControl *control, gpointer userdata)
 }
 #endif
 
-class Main : public Lum::OS::MainDialog<MyWindow>
+class Main : public Lum::GUIApplication<MyWindow>
 {
 public:
-  Main()
+  bool Initialize()
   {
+    std::wstring appDataDir;
+
     info.SetProgram(Lum::Base::StringToWString(PACKAGE_NAME));
     info.SetVersion(Lum::Base::StringToWString(PACKAGE_VERSION));
     info.SetDescription(L"Where the #*~@# am I?");
@@ -1856,6 +1860,31 @@ public:
     info.SetContact(L"Tim Teulings <tim@teulings.org>");
     info.SetCopyright(L"(c) 2009, Tim Teulings");
     info.SetLicense(L"GNU Public License");
+
+#if defined(APP_DATADIR)
+    Lum::Manager::FileSystem::Instance()->SetApplicationDataDir(Lum::Base::StringToWString(APP_DATADIR));
+#endif
+
+    if (Lum::Manager::FileSystem::Instance()->GetEntry(Lum::Manager::FileSystem::appDataDir,
+                                                       appDataDir)) {
+
+      Lum::Base::Path path;
+
+      path.SetNativeDir(appDataDir);
+      path.AppendDir(L"icons");
+
+      Lum::Images::loader->AddApplicationSearchPath(path);
+    }
+
+    Lum::Images::loader->SetNormalImagePostfix(L"");
+
+    Lum::Base::Path path;
+
+    path.SetDir(L"data");
+    path.AppendDir(L"icons");
+    Lum::Images::loader->AddApplicationSearchPath(path);
+
+    return Lum::GUIApplication<MyWindow>::Initialize();
   }
 };
 
